# -*- coding: utf-8 -*-
"""Tests for downloader.py"""
from threedi_scenario_downloader import downloader
import configparser
import pytest
from requests.exceptions import HTTPError


def test_set_headers():
    config = configparser.ConfigParser()
    config.read("threedi_scenario_downloader/tests/testdata/realconfig.ini")
    downloader.set_headers(
        config["credentials"]["username"], config["credentials"]["password"]
    )


def test_get_headers():
    headers = downloader.get_headers()
    config = configparser.ConfigParser()
    config.read("threedi_scenario_downloader/tests/testdata/realconfig.ini")

    assert headers["username"] == config["credentials"]["username"]
    assert headers["password"] == config["credentials"]["password"]


def test_find_scenario_by_model_slug():
    r = downloader.find_scenarios_by_model_slug(
        "005a58d09538e4e4cdbe48f2f3f22aeb89330ae4"
    )
    assert r[0]["uuid"] == "06c38953-31ec-4f6d-ae1f-ccdf31a348ae"


def test_find_scenario_by_name():
    r = downloader.find_scenarios_by_name("lizardapitest")
    assert r[0]["uuid"] == "06c38953-31ec-4f6d-ae1f-ccdf31a348ae"


def test_get_netcdf_link():
    url = downloader.get_netcdf_link("06c38953-31ec-4f6d-ae1f-ccdf31a348ae")
    assert url == "https://demo.lizard.net/api/v3/scenario-results/52331/results_3di.nc"


def test_get_raster():
    raster = downloader.get_raster(
        "06c38953-31ec-4f6d-ae1f-ccdf31a348ae", "depth-max-dtri"
    )
    assert raster["uuid"] == "4ef95627-e370-4eba-a5bc-bed661a3101a"


def test_get_raster_temporal():
    raster = downloader.get_raster("06c38953-31ec-4f6d-ae1f-ccdf31a348ae", "depth-dtri")
    assert raster["uuid"] == "907132e8-931a-4b78-8cb1-bad52500be7a"


def test_get_raster_from_non_existing_scenario():
    with pytest.raises(HTTPError):
        raster = downloader.get_raster(
            "06c37953-31ec-4f6d-ae1f-ccdf31a348ae", "depth-max-dtri"
        )
        print(raster)


def test_create_raster_task():
    task = downloader.create_raster_task(
        downloader.get_raster("06c38953-31ec-4f6d-ae1f-ccdf31a348ae", "depth-max-dtri"),
        "EPSG:28992",
        "1000",
    )
    assert task is not None



# -*- coding: utf-8 -*-
"""Tests for downloader.py"""
from threedi_scenario_downloader import downloader
import configparser


def test_set_headers():
    config = configparser.ConfigParser()
    config.read("threedi_scenario_downloader/tests/testdata/realconfig.ini")
    downloader.set_headers(
        config["credentials"]["username"], config["credentials"]["password"]
    )


def test_download_maximum_waterdepth_raster():
    if True:
        downloader.download_maximum_waterdepth_raster(
            "06c38953-31ec-4f6d-ae1f-ccdf31a348ae",
            "EPSG:28992",
            resolution=1000,
            bounds=None,
            pathname="threedi_scenario_downloader/tests/testdata/max_waterdepth.tif",
        )


def test_download_waterdepth_raster():
    if True:
        downloader.download_waterdepth_raster(
            "a790f33c-9f9f-442a-bd7f-c00f77c37bbd",
            "EPSG:28992",
            1000,
            "2018-06-02T06:00:00Z",
            None,
            "threedi_scenario_downloader/tests/testdata/waterdepth.tif",
        )


def test_clear_inbox():
    result = downloader.clear_inbox()
    assert result



# -*- coding: utf-8 -*-
"""TODO Docstring, used in the command line help text."""
import argparse
import logging

# import sys

logger = logging.getLogger(__name__)


def get_parser():
    """ Return argument parser. """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        dest="verbose",
        default=False,
        help="Verbose output",
    )
    # add arguments here
    # parser.add_argument(
    #     'path',
    #     metavar='FILE',
    # )
    return parser


def main():
    """ Call command with args from parser. """
    options = get_parser().parse_args()
    if options.verbose:
        log_level = logging.DEBUG
    else:
        log_level = logging.INFO
    logging.basicConfig(level=log_level, format="%(levelname)s: %(message)s")

    try:
        print("Call some function from another file here")
        # ^^^ TODO: pass in options.xyz where needed.
    except:
        logger.exception("An exception has occurred.")
        return 1


if __name__ == "__main__":
    exit(main())



# -*- coding: utf-8 -*-
"""Tests for script.py"""
from threedi_scenario_downloader import scripts


def test_get_parser():
    parser = scripts.get_parser()
    # As a test, we just check one option. That's enough.
    options = parser.parse_args()
    assert options.verbose == False



from setuptools import setup

version = "0.2"

long_description = "\n\n".join([open("README.rst").read(), open("CHANGES.rst").read()])

install_requires = ["requests"]

tests_require = ["pytest", "mock", "pytest-cov", "pytest-flakes", "pytest-black"]

setup(
    name="threedi-scenario-downloader",
    version=version,
    description="Tools for downloading results for 3Di scenarios",
    long_description=long_description,
    # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
    classifiers=["Programming Language :: Python", "Framework :: Django"],
    keywords=[],
    author="Emiel Verstegen",
    author_email="emiel.verstegen@nelen-schuurmans.nl",
    url="https://github.com/nens/threedi-scenario-downloader",
    license="MIT",
    packages=["threedi_scenario_downloader"],
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require={"test": tests_require},
    entry_points={
        "console_scripts": [
            "run-threedi-scenario-downloader = threedi_scenario_downloader.scripts:main"
        ]
    },
)



# -*- coding: utf-8 -*-
"""The downloader part of the threedi_scenario_downloader supplies the user with often used functionality to look up and export 3Di results using the Lizard API"""
import requests
from urllib.parse import urlparse
from time import sleep
import logging
import os

LIZARD_URL = "https://demo.lizard.net/api/v3/"
RESULT_LIMIT = 10
REQUESTS_HEADERS = {}

logging.basicConfig(level=logging.DEBUG)
log = logging.getLogger()


def get_headers():
    return REQUESTS_HEADERS


def set_headers(username, password):
    REQUESTS_HEADERS["username"] = username
    REQUESTS_HEADERS["password"] = password
    REQUESTS_HEADERS["Content-Type"] = "application/json"


def find_scenarios_by_model_slug(model_uuid):
    """Find all 3Di scenario's produced by the supplied model. User needs to be authorized to view the model results."""
    url = "{}scenarios/?model_name__icontains={}&limit={}".format(
        LIZARD_URL, model_uuid, RESULT_LIMIT
    )
    r = requests.get(url=url, headers=get_headers())
    r.raise_for_status()
    return r.json()["results"]


def find_scenarios_by_name(name):
    """Find all 3Di scenario's matching the supplied name that the user is authorized for"""
    url = "{}scenarios/?name__icontains={}&limit={}".format(
        LIZARD_URL, name, RESULT_LIMIT
    )
    r = requests.get(url=url, headers=get_headers())
    r.raise_for_status()
    return r.json()["results"]


def get_netcdf_link(scenario_uuid):
    """Returns the link to the netcdf file of the supplied scenario"""
    r = requests.get(
        url="{}scenarios/{}".format(LIZARD_URL, scenario_uuid), headers=get_headers()
    )
    r.raise_for_status()
    for result in r.json()["result_set"]:
        if result["result_type"]["code"] == "results-3di":
            url = result["attachment_url"]
            return url


def get_raster(scenario_uuid, raster_code):
    """Returns the raster json object given the scenario-uuid and the requested raster type"""
    r = requests.get(
        url="{}scenarios/{}".format(LIZARD_URL, scenario_uuid), headers=get_headers()
    )
    r.raise_for_status()
    for result in r.json()["result_set"]:
        if result["result_type"]["code"] == raster_code:
            return result["raster"]


def create_raster_task(raster, target_srs, resolution, bounds=None, time=None):
    """
    Create a task on the Lizard server to prepare a raster with given SRS and resolution
    Full extent will be used if the bounds are not supplied
    
    """
    if bounds == None:
        bounds = raster["spatial_bounds"]

    e = bounds["east"]
    w = bounds["west"]
    n = bounds["north"]
    s = bounds["south"]

    source_srs = "EPSG:4326"

    bbox = "POLYGON+(({}+{},{}+{},{}+{},{}+{},{}+{}))".format(
        w, n, e, n, e, s, w, s, w, n
    )

    if time is None:
        # non temporal raster
        url = "{}rasters/{}/data/?cellsize={}&geom={}&srs={}&target_srs={}&format=geotiff&async=true".format(
            LIZARD_URL, raster["uuid"], resolution, bbox, source_srs, target_srs
        )
    else:
        # temporal rasters
        url = "{}rasters/{}/data/?cellsize={}&geom={}&srs={}&target_srs={}&time={}&format=geotiff&async=true".format(
            LIZARD_URL, raster["uuid"], resolution, bbox, source_srs, target_srs, time
        )
    r = requests.get(url=url, headers=get_headers())
    r.raise_for_status()
    return r.json()


# From here untested methods are added
def get_task_status(task_uuid):
    """Returns either SUCCES, PENDING or .......?"""
    url = "{}tasks/{}/".format(LIZARD_URL, task_uuid)
    r = requests.get(url=url, headers=get_headers())
    r.raise_for_status()
    return r.json()["task_status"]


def get_task_download_url(task_uuid):
    """In case the task is a succes, return the url to the file in order to download"""
    if get_task_status(task_uuid) == "SUCCESS":
        url = "{}tasks/{}/".format(LIZARD_URL, task_uuid)
        r = requests.get(url=url, headers=get_headers())
        r.raise_for_status()
        return r.json()["result_url"]
    # What to do if task is not a success?


def download_file(url, path):
    """
    Download a file (url) to the specified path
    Example: download_file(http://whatever.com/file.txt,os.path.normpath('somefolder/somefile.txt')) 
    """
    r = requests.get(url)
    r.raise_for_status()
    with open(path, "wb") as file:
        for chunk in r.iter_content(100000):
            file.write(chunk)


def download_task(task_uuid, pathname=None):
    """
    Seaches for url belonging to task_uuid and downloads this file to the specified filelocation and name.
    If no pathname is supplied the url path will be used as a relative path to store the data.
    """
    if get_task_status(task_uuid) == "SUCCESS":
        download_url = get_task_download_url(task_uuid)
        if pathname == None:
            pathname = os.path.basename(urlparse(download_url).path)
        download_file(download_url, pathname)


def download_raster(
    scenario_uuid,
    raster_code,
    target_srs,
    resolution,
    bounds=None,
    time=None,
    pathname=None,
):
    """
    Method looks up the maximum waterdepth raster based on scenario_uuid, creates a task and downloads task when succesfull
    """
    raster = get_raster(scenario_uuid, raster_code)
    task = create_raster_task(raster, target_srs, resolution, bounds, time)
    task_uuid = task["task_id"]

    log.debug("Start waiting for task {} to finish".format(task_uuid))
    while get_task_status(task_uuid) == "PENDING":
        sleep(5)
        log.debug("Still waiting for task {}".format(task_uuid))

    if get_task_status(task_uuid) == "SUCCESS":
        # task is a succes, return download url
        log.debug(
            "Task succeeded, start downloading url: {}".format(
                get_task_download_url(task_uuid)
            )
        )
        print(
            "Task succeeded, start downloading url: {}".format(
                get_task_download_url(task_uuid)
            )
        )
        download_task(task_uuid, pathname)
    else:
        log.debug("Task failed")


def download_maximum_waterdepth_raster(
    scenario_uuid, target_srs, resolution, bounds=None, pathname=None
):
    """Complete procedure to download a 3Di maximum waterdepth raster"""
    download_raster(
        scenario_uuid, "depth-max-dtri", target_srs, resolution, bounds, None, pathname
    )


def download_total_damage_raster(
    scenario_uuid, target_srs, resolution, bounds=None, pathname=None
):
    """Complete procedure to download a 3Di total damage raster"""
    download_raster(
        scenario_uuid, "3di_damage", target_srs, resolution, bounds, None, pathname
    )


def download_waterdepth_raster(
    scenario_uuid, target_srs, resolution, time, bounds=None, pathname=None
):
    """Complete procedure to download a snapshot of a temporal 3Di waterdepth raster"""
    download_raster(
        scenario_uuid,
        "depth-dtri",
        target_srs,
        resolution,
        bounds=bounds,
        time=time,
        pathname=pathname,
    )


def clear_inbox():
    url = "{}inbox/".format(LIZARD_URL)
    r = requests.get(url=url, headers=get_headers())
    r.raise_for_status()
    messages = r.json()["results"]
    for msg in messages:
        msg_id = msg["id"]
        read_url = "{}inbox/{}/read/".format(LIZARD_URL, msg_id)
        r = requests.post(url=read_url, headers=get_headers())
        r.raise_for_status()
    return True
